function SolveBigModel
    % This function runs the multi-scale agent-based B cell model.
    % Parameters for model simulations are inputed below. See "Developing a
    % multi-scale understaning of the B cell immune response" Ph.D. Thesis
    % by Maxim Shokhirev or the paper "A multi-scale approach reveals that 
    % NFkB enforces a decision to divide or die in B cells." Shokhirev et
    % al. 2014. Submitted to Nature Molecular Systems Biology. 
    fclose all;        
    clear all;
  
   
    maxGens = 9;
    %maximum time for simulations
    maxTime=144*60;  % minutes
    N=250;  % starting cells
    replicates = 2;
    % constant stimulus since unbound ligand is outside the cell
    %c = clock;
    % now lets run all of the cells repeatedly and 

    %if  matlabpool('size') > 0
     %   matlabpool close;
    %end
    if matlabpool('size') == 0
        matlabpool 12;
    end

 
    paramsToRun = {};
    paramNames = {};

     %%%%%%%%%%%%% Parameter DESCRIPTION %%%%%%%%%%%%%%
     % NFKB_CYCD_T NFKB_BCL2_T  NFKB_MYC_T    GROWTH_T  
     % RATE_CV CONC_CV PARTITION_CV VOL_CV IKK_CV  
     % GMSyn  GMDeg  MassGrowth MassDecay 
     % CYCDtSyn CYCDtDeg BCL2Syn BCL2tDeg 
     % MYCSyn MYCDeg mMycSyn mMycDeg L
     % IKK STIMULUS, Crel monomer,cRelFeedback,eps (Ribosome modifier)
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%      
    parameters= {'CD_T';'BCL_T';'MYC_T';'GROW_T';'RCV';'CCV';
        'PCV';'VCV';'IKKCV';'GMSyn';'GMDeg';'MGrow';'MDeg';'CDtSyn';'CDtDeg';
        'BCLtSyn';'BCLtDeg';'MYCSyn';'MYCDeg';'MYCtSyn';'MYCtDeg';'L';
        'IKK Fxn';'Crel?';'NFkBFbck';'EPS';'IKKTotal'};
    
    % Running this m file will produce the WT250 nM solution. Please adjust
    % matlabpool to the number of cores available (maximum 12). If you are
    % interested in running other conditions. Please uncomment the
    % parameters of interest below. 
    
    paramsToRun{end+1}=[40;40;40;40;         % CYCDT BCLT MYCT GROWT
        0.1;0.25;0.07 ;0.05 ; 0.25 ;    % RCV CCV PCV VCV IKKCV
        0.002; 0.0002 ;0.004; 0.002;     % GMSyn GMDeg MGrow MDeg
        
        0.0030;0.002;13;0.007;            % CYCDtSyn CYCDtDeg BCL2tSyn BCL2tDeg 
        0.0040;0.0020;1;0.0231;1;         % MYCSyn MYCDeg mMycSyn mMycDeg L
        1;1;10;1;140];                         % IKK input (see intial_parameters8.m), cRel present?, NFkB induction modifier, translation modifier, IKK total (nM)
     paramNames{end+1}='WT250NewIKK2';
   
%     paramsToRun{end+1}=[40;40;40;40;         % CYCDT BCLT MYCT GROWT
%         0.1;0.25;0.07 ;0.05 ; 0.25 ;    % RCV CCV PCV VCV IKKCV
%         0.002; 0.0002 ;0.004; 0.002;     % GMSyn GMDeg MGrow MDeg
%         
%         0.0030;0.002;13;0.007;            % CYCDtSyn CYCDtDeg BCL2tSyn BCL2tDeg 
%         0.0040;0.0020;1;0.0231;1;         % MYCSyn MYCDeg mMycSyn mMycDeg L
%         1;1;10;1;140];    
%     paramNames{end+1}='Lymphoma';
%     paramsToRun{end+1}=[40;40;40;40;         % CYCDT BCLT MYCT GROWT
%         0.1;0.25;0.07 ;0.05 ; 0.25 ;    % RCV CCV PCV VCV IKKCV
%         0.002; 0.0002 ;0.004; 0.002;     % GMSyn GMDeg MGrow MDeg
%         
%         0.0030;0.002;13;0.007;            % CYCDtSyn CYCDtDeg BCL2tSyn BCL2tDeg 
%         0.0040;0.0020;1;0.0231;1;         % MYCSyn MYCDeg mMycSyn mMycDeg L
%         1;1;10;1;140/2];    
%      paramNames{end+1}='IKKInhibLymphoma';
%      paramsToRun{end+1}=[40;40;40;40;         % CYCDT BCLT MYCT GROWT
%         0.1;0.25;0.07 ;0.05 ; 0.25 ;    % RCV CCV PCV VCV IKKCV
%         0.002; 0.0002 ;0.004; 0.002;     % GMSyn GMDeg MGrow MDeg
%         
%         0.0030;0.002;13;0.007;            % CYCDtSyn CYCDtDeg BCL2tSyn BCL2tDeg 
%         0.0040;0.0020;1;0.0231;1;         % MYCSyn MYCDeg mMycSyn mMycDeg L
%         1;1;10;0.7;140];    
%      paramNames{end+1}='RapLymphoma';
%      
%      paramsToRun{end+1}=[40;40;40;40;         % CYCDT BCLT MYCT GROWT
%         0.1;0.25;0.07 ;0.05 ; 0.25 ;    % RCV CCV PCV VCV IKKCV
%         0.002; 0.0002 ;0.004; 0.002;     % GMSyn GMDeg MGrow MDeg
%         
%         0.0030;0.002;13;0.007;            % CYCDtSyn CYCDtDeg BCL2tSyn BCL2tDeg 
%         0.0040;0.0020;1;0.0231;1;         % MYCSyn MYCDeg mMycSyn mMycDeg L
%         1;1;12.5;1];    
%      paramNames{end+1}='WT250HighCrel';
%      paramsToRun{end+1}=[40;40;40;40;         % CYCDT BCLT MYCT GROWT
%         0.2;0.5;0.07 ;0.05 ; 0.25 ;    % RCV CCV PCV VCV IKKCV
%         0.002; 0.0002 ;0.004; 0.002;     % GMSyn GMDeg MGrow MDeg
%         
%         0.0030;0.002;13;0.007;            % CYCDtSyn CYCDtDeg BCL2tSyn BCL2tDeg 
%         0.0040;0.0020;1;0.0231;1;         % MYCSyn MYCDeg mMycSyn mMycDeg L
%         1;1;10;1];    
%      paramNames{end+1}='WT250DoubleProtVar';
%      paramsToRun{end+1}=[40;40;40;40;         % CYCDT BCLT MYCT GROWT
%         0.1;0.25;0.07 ;0.05 ; 0.25 ;    % RCV CCV PCV VCV IKKCV
%         0.002; 0.0002 ;0.004; 0.002;     % GMSyn GMDeg MGrow MDeg
%         
%         0.0030;0.002;13;0.007;            % CYCDtSyn CYCDtDeg BCL2tSyn BCL2tDeg 
%         0.0040;0.0020;1;0.0231;1;         % MYCSyn MYCDeg mMycSyn mMycDeg LL
%         1;0;10;1];    
%      paramNames{end+1}='CKO250NewIKK';
%      paramsToRun{end+1}=[40;40;40;40;         % CYCDT BCLT MYCT GROWT
%         0.1;0.25;0.07 ;0.05 ; 0.25 ;    % RCV CCV PCV VCV IKKCV
%         0.002; 0.0002 ;0.004; 0.002;     % GMSyn GMDeg MGrow MDeg
%         
%         0.0030;0.002;13;0.007;            % CYCDtSyn CYCDtDeg BCL2tSyn BCL2tDeg 
%         0.0040;0.0020;1;0.0231;1;         % MYCSyn MYCDeg mMycSyn mMycDeg LL
%         2;1;10;1];    
%      paramNames{end+1}='WT10NewIKK';
%          paramsToRun{end+1}=[40;40;40;40;         % CYCDT BCLT MYCT GROWT
%         0.1;0.25;0.07 ;0.05 ; 0.25 ;    % RCV CCV PCV VCV IKKCV
%         0.002; 0.0002 ;0.004; 0.002;     % GMSyn GMDeg MGrow MDeg
%         
%         0.0030;0.002;13;0.007;            % CYCDtSyn CYCDtDeg BCL2tSyn BCL2tDeg 
%         0.0040;0.0020;1;0.0231;1;         % MYCSyn MYCDeg mMycSyn mMycDeg LL
%         1;1;10;0.7];    
%      paramNames{end+1}='RapNewIKK';
%      paramsToRun{end+1}=[40;40;40;40;         % CYCDT BCLT MYCT GROWT
%         0.05;0.125;0.07 ;0.05 ; 0.25 ;    % RCV CCV PCV VCV IKKCV
%         0.002; 0.0002 ;0.004; 0.002;     % GMSyn GMDeg MGrow MDeg
%         
%         0.0030;0.002;13;0.007;            % CYCDtSyn CYCDtDeg BCL2tSyn BCL2tDeg 
%         0.0040;0.0020;1;0.0231;1;         % MYCSyn MYCDeg mMycSyn mMycDeg LL
%         1;1;10;1];    
%      paramNames{end+1}='HalfProtNewIKK';
%      paramsToRun{end+1}=[40;40;40;40;         % CYCDT BCLT MYCT GROWT
%         0.1;0.25;0.07 ;0.05 ; 0.125 ;    % RCV CCV PCV VCV IKKCV
%         0.002; 0.0002 ;0.004; 0.002;     % GMSyn GMDeg MGrow MDeg
%         
%         0.0030;0.002;13;0.007;            % CYCDtSyn CYCDtDeg BCL2tSyn BCL2tDeg 
%         0.0040;0.0020;1;0.0231;1;         % MYCSyn MYCDeg mMycSyn mMycDeg LL
%         1;1;10;1];    
%      paramNames{end+1}='HalfIKKNewIKK';
%      paramsToRun{end+1}=[40;40;40;40;         % CYCDT BCLT MYCT GROWT
%         0.1;0.25;0.00 ;0.00 ; 0.25 ;    % RCV CCV PCV VCV IKKCV
%         0.002; 0.0002 ;0.004; 0.002;     % GMSyn GMDeg MGrow MDeg
%         0.0030;0.002;13;0.007;            % CYCDtSyn CYCDtDeg BCL2tSyn BCL2tDeg 
%         0.0040;0.0020;1;0.0231;1;         % MYCSyn MYCDeg mMycSyn mMycDeg LL
%         1;0;10;1];   
%     paramNames{end+1}='NoSizeNewIKK';
%          paramsToRun{end+1}=[40;40;40;40;         % CYCDT BCLT MYCT GROWT
%         0.05;0.125;0.035 ;0.025 ; 0.125;    % RCV CCV PCV VCV IKKCV
%         0.002; 0.0002 ;0.004; 0.002;     % GMSyn GMDeg MGrow MDeg
%         
%         0.0030;0.002;13;0.007;            % CYCDtSyn CYCDtDeg BCL2tSyn BCL2tDeg 
%         0.0040;0.0020;1;0.0231;1;         % MYCSyn MYCDeg mMycSyn mMycDeg LL
%         1;0;10;1];   
%     paramNames{end+1}='HalfExtrinsicNewIKK';
%         paramsToRun{end+1}=[40;40;40;40;         % CYCDT BCLT MYCT GROWT
%         0.1;0.25;0.07 ;0.05 ; 0.25 ;    % RCV CCV PCV VCV IKKCV
%         0.002; 0.0002 ;0.004; 0.002;     % GMSyn GMDeg MGrow MDeg
%         
%         0.0030;0.002;13;0.007;            % CYCDtSyn CYCDtDeg BCL2tSyn BCL2tDeg 
%         0.0040;0.0020;1;0.0231;1;         % MYCSyn MYCDeg mMycSyn mMycDeg LL
%         1;1;10;1];    
%      paramNames{end+1}='WT250NewIKK2';

            tic;
    user = memory;
    fprintf(1,'Memory Available: %3.3f gigs\n',(user.MemAvailableAllArrays)/(1024*1024*1024));
    for k = 1:numel(paramsToRun)
        experiment = randomCondition(maxTime,paramsToRun{k});
        solutions ={};

        for rep = 1:replicates
            label = sprintf('%s_%d',paramNames{k},rep);
            for i = 1:length(parameters)
                label= sprintf('%s %s=%2.2f',label,parameters{i},paramsToRun{k}(i));
            end
            fprintf(1,'%s\n',label);
            solutions{end+1}=runSolution(experiment,N,maxTime,maxGens);      
        end
        saveSolutions(solutions,N,maxTime,maxGens,paramNames{k},label);

    end
    toc


  % matlabpool close;
    clear all;
    
end

function sol= runSolution(experiment, N, maxTime, maxGens)
    SEE = 0;
    sol = duplicate(experiment);    
    vo = initial_parameters8(experiment);
    %total cell counts as a function of time
    sol.counts=ones(ceil(maxTime/30),maxGens);
    sol.fullCounts = zeros(maxTime,maxGens); % total count as a function of gen and time
    sol.integralCounts = zeros(maxGens); % total number of generation cells
    sol.h24counts = zeros(maxGens); % total number of generation cells
    %all of the cells currently waiting to be computed
    cells = cell(N,1);

    %average concentrations over time
    sol.raverages = zeros(100,144,maxGens);
    sol.qaverages = zeros(100,144,maxGens);
    sol.Tdivs= zeros(maxGens,ceil(maxTime/60)); % in terms of hours
    sol.Tdecides = zeros(maxGens,ceil(maxTime/60));
    sol.Tdies= zeros(maxGens,ceil(maxTime/60));
    sol.TdivTimes = cell(1);
    sol.TdieTimes = cell(1);
    sol.TdecideTimes = cell(1);
    sol.TdivMothers = cell(1);
    sol.TdieMothers = cell(1);
    sol.TdecideMothers = cell(1);    
    for i = 1:maxGens
        sol.TdivTimes{end+1}=cell(1);
        sol.TdieTimes{end+1}=cell(1);
        sol.TdecideTimes{end+1}=cell(1);
        sol.TdivMothers{end+1}=cell(1);
        sol.TdieMothers{end+1}=cell(1);
        sol.TdecideMothers{end+1}=cell(1);
    end

    sol.divbag = {};
    sol.diebag ={};
    sol.decidebag = {};
    TdivFits = zeros(maxGens,6); %lognormal fits: E[Tdiv]fit, s.d.[Tdiv]fit, E[Tdiv]min, E[Tdiv]max, s.d.[Tdiv]min, s.d.[Tdiv]max
    TdieFits = zeros(maxGens,6);
    sol.gs = zeros(maxGens); % fraction that grow to larger than v0+1.75 their starting size or to a size of 4 (both dividers and non-dividers)
    sol.fs = zeros(maxGens); % fraction that divides (assume that the rest will eventually die)
    sol.tp = zeros(maxGens); % fraction that grows and divides
    sol.fp = zeros(maxGens); % fraction that grows but dies
    sol.tn = zeros(maxGens); % fraction that doesn't grow and dies
    sol.fn = zeros(maxGens); % fraction that doesn't grow but divides
    sol.tp24 = zeros(maxGens); % fraction that grows and divides 24
    sol.fp24 = zeros(maxGens); % fraction that grows but dies 24
    sol.tn24 = zeros(maxGens); % fraction that doesn't grow and dies 24
    sol.fn24 = zeros(maxGens); % fraction that doesn't grow but divides 24
    optionsDiv  = odeset('AbsTol',1E-5,'RelTol',1E-3,'Events',@division);
    fprintf(1,'Initializing: %d cells\n',N);
    parfor i=1:N  
        dividing    = true;    % True if at equilibrium
        %%%%% RUN TO INITIALIZE THE PERSISTENT VARS %%%%%%%%%%
        v = initial_parameters8(experiment);
        nfkbDivDeath_ode_cRel([],[],[],v);
        
        v.SPECIES{1}(62) = 0;
        v.k2(77)= 1; 
        %%%%% RUN TO ACHIEVE STEADY STATE WITH BASAL SIGNALING %%%%%%%%%%%%%%
        dcount = 0;
        while dividing
            [t,x,te,ye,ie]=ode15s(@(t,x) nfkbDivDeath_ode_cRel(t,x,[],v), [0 24*60], v.SPECIES{1},optionsDiv);
            if ie ==1
                %division occurred
                x(end,(numel(v.SPECIES{1})-v.CELL_CYCLE+15))=x(end,(numel(v.SPECIES{1})-v.CELL_CYCLE+15))/2;
                x(end,(numel(v.SPECIES{1})-v.CELL_CYCLE+13))=x(end,(numel(v.SPECIES{1})-v.CELL_CYCLE+13))/2;
                x(end,126)=1; % to avoid jumping back weirdness with cdh1
                dcount=dcount+1;
                if dcount > 1
                    fprintf(1,'Extra division with tdiv=%f\n',t(end));
                end
            else
                dividing = false;
            end
            v.SPECIES{1} = x(end,:);
            v.OUTPUT1 = [v.OUTPUT1;x];
        end
        v.SPECIES{1} = v.OUTPUT1(end,:)';
        v = getRandomCell(v);
        if experiment.VOL_CV > 0
            factor = max(0,normrnd(1,experiment.VOL_CV));
            v.SPECIES{1}(134)=v.SPECIES{1}(134)*factor;
            v.SPECIES{1}(132) =v.SPECIES{1}(132)*factor;
        end
        v.SPECIES{1}(62) = experiment.L;
        v.k2(77) = experiment.EPS;
%        v.SPECIES{1}(63:119) = vo.SPECIES{1}(63:119);
%       v.SPECIES{1}(143:144) = vo.SPECIES{1}(143:144);
        cells{i}.v=v;
        cells{i}.v.PHASE = 2;
        cells{i}.x = cells{i}.v.SPECIES{1};
        %cell.x(49)=L;
        cells{i}.birthday=0;  % start at 0 hours
        cells{i}.gen = 1;  % undivided
        cells{i}.id =i;
        cells{i}.parent =0;
    end
    cell_id=N+1;

    while ~isempty(cells)
        stridelength=ceil(length(cells)/100);
        fprintf(1,'\nProcessing %d cells\n',length(cells));
        for stride=1:stridelength
            celllength = min(100,length(cells)-(stride-1)*100);
            parfor temporary=((stride-1)*100+1):((stride-1)*100)+celllength                   
                cells{temporary}.v.SIM_TIME=max(1,(maxTime)-cells{temporary}.birthday);
                cells{temporary}.v.START_TIME = cells{temporary}.birthday;
                cells{temporary}.v=run_cell(cells{temporary}.v);
                tdiv = 999999;
                tdie = 999999;
                tdecide=999999;
                if cells{temporary}.v.EVENT>1    % the cell must have died
                    tdie=cells{temporary}.v.SIM_TIME;    
                elseif cells{temporary}.v.EVENT==1
                    tdiv=cells{temporary}.v.SIM_TIME;
                end
                cells{temporary}.grew=0;
                if numel(cells{temporary}.v.OUTPUT2) > 0 && (cells{temporary}.v.OUTPUT2(end,134) > cells{temporary}.v.OUTPUT2(1,134)+2 || cells{temporary}.v.OUTPUT2(end,134) > 4.7)
                    tdecide=find(cells{temporary}.v.OUTPUT2(:,134)>cells{temporary}.v.OUTPUT2(1,134)*1.05,1,'first');
                    cells{temporary}.grew = 1;
                end
                cells{temporary}.x = cells{temporary}.v.OUTPUT2(end,:);   % save the last concentrations
                cells{temporary}.xc = cells{temporary}.v.OUTPUT2;   % save the entire trajectory?
                cells{temporary}.short = cells{temporary}.v.SHORTOUTPUT;
                cells{temporary}.tdiv = tdiv;
                cells{temporary}.tdie = tdie;
                cells{temporary}.tdecide= tdecide;
            end
        end
        cells2={};    
        cell_count = 0;
        real_count = 0;
        a_mass = 0;
        a_cparp=0;
        a_CD=0;
        a_GM=0;
        a_cRel=0;
        a_RelA=0;
        a_myc=0;
        a_bcl2=0;
        numdivs = 0;
        numdeaths = 0;
        numdecides = 0;
        numRelAPositive = 0;
        numcRelPositive = 0;
        sol.deaths = [];
        sol.divs = [];
        sol.decides = [];
        dividers = 0;
        nondividers = 0;
        Cgen = 0;
        while ~isempty(cells)
            theCell = cells{1};
            a_mass=a_mass+theCell.x(134);
            a_cparp = a_cparp+theCell.x(84);
            a_CD = a_CD+theCell.x(122);
            a_GM = a_GM+theCell.x(132);
            a_RelA = a_RelA+theCell.x(33);
            a_cRel = a_cRel+theCell.x(53);
            a_myc=a_myc+theCell.x(143);
            a_bcl2=a_bcl2+theCell.x(94);
            if max(theCell.v.OUTPUT2(:,33)) > sol.NFKB_CYCD_T/2
                numRelAPositive = numRelAPositive+1;
            end
            if max(theCell.v.OUTPUT2(:,53)) > sol.NFKB_CYCD_T/2
                numcRelPositive = numcRelPositive+1;
            end
            if SEE == 1;
                figure(1);
                subplot(3,3,theCell.gen);
                hold on;
                plot(theCell.xc(:,134));
                figure(2);
                subplot(3,3,theCell.gen);
                hold on;
                plot(theCell.xc(:,53));
                figure(3);
                subplot(3,3,theCell.gen);
                hold on;
                plot(theCell.xc(:,143));
            end
            real_count=real_count+1;
            if theCell.x(134) > 10.0
                fprintf('Cell had ending mass of %3.3f\n',theCell.x(134));
            end
            sol.integralCounts(theCell.gen)=sol.integralCounts(theCell.gen)+1;
            bindex=ceil(theCell.birthday); % birth index
            index = ceil(theCell.v.SIM_TIME);    % length of the time course
            Cgen = theCell.gen;
            for i=((bindex)+1):30:index+bindex % for each time point in the timecourse
                if i-bindex > 0 && i <= maxTime  
                    sol.counts(ceil(i/30),theCell.gen)=sol.counts(ceil(i/30),theCell.gen)+1; % add the counts to the summary
                end
            end
            for i=((bindex)+1):1:(index+bindex) % for each time point in the timecourse
                if i-bindex > 0 && i <= maxTime  
                    sol.fullCounts(i,theCell.gen)=sol.fullCounts(i,theCell.gen)+1; % add the counts to the summary
                end
            end
            if theCell.tdecide <= maxTime;
                sol.Tdecides(theCell.gen,ceil(theCell.tdecide/60))=sol.Tdecides(theCell.gen,ceil(theCell.tdecide/60))+1;
                sol.decides(end+1)=ceil(theCell.tdecide/60);
                sol.gs(theCell.gen)=sol.gs(theCell.gen)+1;
                numdecides=numdecides+1;
            end

            if theCell.tdie < theCell.tdiv && theCell.tdie+1+bindex < maxTime
                % cell died
                %fprintf(1,'A cell died at time %f it was %f minutes old. Total cells %d\n',theCell.tdie+theCell.birthday,theCell.tdie,length(cells)+length(cells2));
                numdeaths= numdeaths+1;
                sol.Tdies(theCell.gen,ceil(theCell.tdie/60))=sol.Tdies(theCell.gen,ceil(theCell.tdie/60))+1;
                sol.deaths(end+1)=ceil(theCell.tdie/60);
                if theCell.grew
                    sol.fp(theCell.gen) = sol.fp(theCell.gen)+1;
                    if (theCell.tdie < 36*60 &&theCell.tdie > 12*60) || (theCell.gen == 1 && (theCell.tdie < 48*60 &&theCell.tdie > 24*60))
                        sol.fp24(theCell.gen) = sol.fp24(theCell.gen) +1;
                        sol.h24counts(theCell.gen) = sol.h24counts(theCell.gen) +1;
                    end
                    sol.TdecideTimes{theCell.gen}{end+1}=theCell.tdecide;
                    sol.TdecideMothers{theCell.gen}{end+1}=theCell.parent;
                else
                    sol.tn(theCell.gen) = sol.tn(theCell.gen)+1;
                    if (theCell.tdie < 36*60 &&theCell.tdie > 12*60) || (theCell.gen == 1 && (theCell.tdie < 48*60 &&theCell.tdie > 24*60))
                        sol.tn24(theCell.gen) = sol.tn24(theCell.gen) +1;
                        sol.h24counts(theCell.gen) = sol.h24counts(theCell.gen) +1;
                    end
                end
                for s=1:144
                    if size(theCell.v.OUTPUT2,1) > 100
                        for i = 1:1:100
                            sol.qaverages(i,s,theCell.gen)=sol.qaverages(i,s,theCell.gen)+theCell.v.OUTPUT2(ceil((size(theCell.v.OUTPUT2,1)*i)/100),s);
                        end
                    else
                        for i = 1:1:100
                            j=ceil((i*size(theCell.v.OUTPUT2,1))/100.0);
                            sol.qaverages(i,s,theCell.gen)=sol.qaverages(i,s,theCell.gen)+theCell.v.OUTPUT2(j,s);
                        end
                    end
                end
                sol.TdieTimes{theCell.gen}{end+1}=theCell.tdie;
                sol.TdieMothers{theCell.gen}{end+1}=theCell.parent;
                nondividers=nondividers+1;
                cells(1)=[];
            elseif theCell.tdiv+1+bindex < maxTime
                %cell divided
                cell_count=cell_count+1;
                if theCell.grew
                    sol.tp(theCell.gen) = sol.tp(theCell.gen)+1;
                    if (theCell.tdiv < 36*60 &&theCell.tdiv > 12*60) || (theCell.gen == 1 && (theCell.tdiv < 48*60 &&theCell.tdiv > 24*60))
                        sol.tp24(theCell.gen) = sol.tp24(theCell.gen) +1;
                        sol.h24counts(theCell.gen) = sol.h24counts(theCell.gen) +1;
                    end
                    sol.TdecideTimes{theCell.gen}{end+1}=theCell.tdecide;
                    sol.TdecideMothers{theCell.gen}{end+1}=theCell.parent;
                else
                    sol.fn(theCell.gen) = sol.fn(theCell.gen)+1;
                    if theCell.tdiv < 24*60 || (theCell.gen == 1 && theCell.tdiv < 48*60)
                        sol.fn24(theCell.gen) = sol.fn24(theCell.gen) +1;
                        sol.h24counts(theCell.gen) = sol.h24counts(theCell.gen) +1;
                    end
                end
                sol.fs(theCell.gen)=sol.fs(theCell.gen)+1;
                %fprintf(1,'A cell divided at time %f it was %f minutes old. Total cells %d\n',theCell.tdiv+theCell.birthday,theCell.tdiv,length(cells)+length(cells2));
                numdivs = numdivs+1;
                sol.Tdivs(theCell.gen,ceil(theCell.tdiv/60))=sol.Tdivs(theCell.gen,ceil(theCell.tdiv/60))+1;
                sol.divs(end+1)=ceil(theCell.tdiv/60);
                for s=1:144
                    if size(theCell.v.OUTPUT2,1) > 100
                        for i = 1:1:100
                            sol.raverages(i,s,theCell.gen)=sol.raverages(i,s,theCell.gen)+theCell.v.OUTPUT2(ceil((size(theCell.v.OUTPUT2,1)*i)/100),s);
                        end
                    else
                        for i = 1:1:100
                            j=ceil((i*size(theCell.v.OUTPUT2,1))/100.0);
                            sol.raverages(i,s,theCell.gen)=sol.raverages(i,s,theCell.gen)+theCell.v.OUTPUT2(j,s);
                        end
                    end
                end
                sol.TdivTimes{theCell.gen}{end+1}=theCell.tdiv;
                sol.TdivMothers{theCell.gen}{end+1}=theCell.parent;

                dividers=dividers+1;
                r = 1;
                if experiment.PARTITION_CV>0
                    r = max(0,normrnd(1,experiment.PARTITION_CV));
                end
                dcell.v = getDaughterCell(theCell.v,vo,r);
                dcell.x = dcell.v.SPECIES{1};
                dcell.x = theCell.xc(end,:);
                dcell.x(62) = experiment.L;
                %however we need to update the cellular machine and size
                dcell.x(134)=theCell.xc(end,134)/2;
                dcell.x(132)=theCell.xc(end,132)/2;
                dcell.x(126)=1.0; % set to 1 to avoid weirdness.       
                %Also need empty out the contents of the nucleus
                %(IkBa,b,d,e,RelA,P50,cRel)
                for i = 3:3:21
                    dcell.x(i-1)= (dcell.x(i-1)+dcell.x(i))/2;
                    dcell.x(i)=(dcell.x(i-1)+dcell.x(i))/2;
                end
                %Empty out the nucleus continued (all dimer
                %combinations)
                for i = 23:2:61
                    dcell.x(i-1)= (dcell.x(i-1)+dcell.x(i))/2;
                    dcell.x(i)=(dcell.x(i-1)+dcell.x(i))/2;
                end
                %update params
                dcell.birthday=theCell.tdiv+theCell.birthday;  % start at division time
                dcell.gen = min(theCell.gen+1,maxGens);
                dcell.id = cell_id;
                cell_id=cell_id+1;
                dcell.parent = theCell.id;
                dcell.v.SPECIES{1} = dcell.x;
                if numel(cells2) < N*100
                    cells2{end+1} = dcell;  % append
                end
                %Daughter 2
                r = ((2*r-1)/r);
                dcell2.v = getDaughterCell(theCell.v,vo,r);
                dcell2.x = dcell2.v.SPECIES{1};
                %however we need to update the cellular machine and size
                dcell2.x(134)=theCell.xc(end,134)/2;
                dcell2.x(132)=theCell.xc(end,132)/2;
                dcell2.x(126) = 1.0; % set to 1 to avoid weirdness
                %Also need empty out the contents of the nucleus
                %(IkBa,b,d,e,RelA,P50,cRel)
                for i = 3:3:21
                    dcell2.x(i-1)= (dcell2.x(i-1)+dcell2.x(i))/2;
                    dcell2.x(i)=(dcell2.x(i-1)+dcell2.x(i))/2;
                end
                %Empty out the nucleus continued (all dimer
                %combinations)
                for i = 23:2:61
                    dcell2.x(i-1)= (dcell2.x(i-1)+dcell2.x(i))/2;
                    dcell2.x(i)=(dcell2.x(i-1)+dcell2.x(i))/2;
                end
                %%% L is constant
                dcell2.x(62) = experiment.L;
                 %update params
                dcell2.birthday=theCell.tdiv+theCell.birthday;  % start at division time
                dcell2.gen = min(theCell.gen+1,maxGens);
                dcell2.id = cell_id;
                cell_id=cell_id+1;
                dcell2.parent = theCell.id;
                dcell2.v.SPECIES{1} = dcell2.x;
                %dcell2.v.OUTPUT2 = [];
                if numel(cells2) < N*100
                    cells2{end+1} = dcell2;  % append
                end
                cells(1)=[];
            else % simulation ended
                cells(1)=[];
            end
        end
        sol.divbag{end+1}=sol.divs;
        sol.diebag{end+1}=sol.deaths;
        sol.decidebag{end+1}=sol.decides;
        for s=1:144
            for i=1:100
                sol.qaverages(i,s,Cgen)=sol.qaverages(i,s,Cgen)/nondividers;
                sol.raverages(i,s,Cgen)=sol.raverages(i,s,Cgen)/dividers;
            end
        end
        cells = cells2;
        fprintf(1,'AMass=%3.3f, AcPARP=%3.3f, ACD=%3.3f, AGM=%3.3f, ARelA=%3.3f, AcRel=%3.3f, Amyc=%3.3f,\nAbcl2=%3.3f Divs=%d, Deaths=%d, Decides=%d,RelA+=%0.3f, cRel+=%0.3f, ATdiv=%3.2f MinTdiv=%3.2f ATdie=%3.2f MinTdie=%3.2f Cells=%d\n',(a_mass/real_count),(a_cparp/real_count),(a_CD/real_count), (a_GM/real_count),(a_RelA/real_count), (a_cRel/real_count),(a_myc/real_count), (a_bcl2/real_count),numdivs,numdeaths,numdecides,(numRelAPositive/real_count),(numcRelPositive/real_count),mean(sol.divs),min(sol.divs),mean(sol.deaths),min(sol.deaths),real_count);
        
    end
    %Post processing of stats
    for i=1:maxGens
        sol.counts(:,i)=sol.counts(:,i)-1;
    end
    accuracies=zeros(maxGens,1);
    logErrors=zeros(maxGens,2);
    statistics=zeros(maxGens,20);
    for g=1:maxGens
        if sol.integralCounts(g) > 0
            %lets compute the stats
            sol.fs(g)=sol.fs(g)/sol.integralCounts(g);
            sol.gs(g)=sol.gs(g)/sol.integralCounts(g);
            sol.tp(g)=sol.tp(g)/sol.integralCounts(g);
            sol.tn(g)=sol.tn(g)/sol.integralCounts(g);
            sol.fp(g)=sol.fp(g)/sol.integralCounts(g);
            sol.fn(g)=sol.fn(g)/sol.integralCounts(g);
            sol.tp24(g)=sol.tp24(g)/sol.h24counts(g);
            sol.tn24(g)=sol.tn24(g)/sol.h24counts(g);
            sol.fp24(g)=sol.fp24(g)/sol.h24counts(g);
            sol.fn24(g)=sol.fn24(g)/sol.h24counts(g);
            statistics(g,1)=sol.integralCounts(g);
            statistics(g,2)=sol.gs(g);
            statistics(g,3)=sol.fs(g);
            statistics(g,4)=sol.tp(g);
            statistics(g,5)=sol.tn(g);
            statistics(g,6)=sol.fp(g);
            statistics(g,7)=sol.fn(g);
            accuracies(g)=(sol.tp(g)+sol.tn(g));
            statistics(g,8)=accuracies(g);
            if ~ isempty(sol.divbag{g})
                sol.maxDivGen=g;
                %lets fit a lognormal to the divs
                vector = sol.divbag{g};
                [parmhat,parmci] = lognfit(vector(:));
                [M,V]=lognstat(parmhat(1),parmhat(2));
                TdivFits(g,1)=M;
                TdivFits(g,2)=sqrt(V);
                [Mlo,Vlo]=lognstat(parmci(1,1),parmci(2,2));
                [Mhi,Vhi]=lognstat(parmci(1,2),parmci(2,1));
                TdivFits(g,3)=Mlo;
                TdivFits(g,4)=Mhi;
                TdivFits(g,5)=sqrt(Vlo);
                TdivFits(g,6)=sqrt(Vhi);
                logErrors(g,1)=(((Mhi-Mlo)/M)+((Vhi-Vlo)/V))/2;
                for i = 1:6
                    statistics(g,8+i)=TdivFits(g,i);
                end
            end
            if ~ isempty(sol.diebag{g}) 
                sol.maxDieGen=g;
                %lets fit a lognormal to the divs
                vector = sol.diebag{g};
                [parmhat,parmci] = lognfit(vector(:));
                [M,V]=lognstat(parmhat(1),parmhat(2));
                TdieFits(g,1)=M;
                TdieFits(g,2)=sqrt(V);
                [Mlo,Vlo]=lognstat(parmci(1,1),parmci(2,2));
                [Mhi,Vhi]=lognstat(parmci(1,2),parmci(2,1));
                TdieFits(g,3)=Mlo;
                TdieFits(g,4)=Mhi;
                TdieFits(g,5)=sqrt(Vlo);
                TdieFits(g,6)=sqrt(Vhi);
                logErrors(g,2)=(((Mhi-Mlo)/M)+((Vhi-Vlo)/V))/2;
                for i = 1:6
                    statistics(g,14+i)=TdieFits(g,i);
                end
            end
        end    
    end
    sol.statistics = statistics;
    sol.logErrors = logErrors;
    sol.TdivFits = TdivFits;
    sol.TdieFits = TdieFits;   
end

function saveSolutions(solutions,N,maxTime,maxGens,label,paramString)
    experiment_label=sprintf('%s %s',label,datestr(now,'mmmm dd, yyyy HH-MM-SS-FFF'));
    mkdir(experiment_label);
    copyfile('*.m',experiment_label);
    ts = 1:30:maxTime;
    ts=ts./60;
    totals = zeros(length(solutions),length(ts));
    for r = 1:length(solutions)
        for t = 1:length(ts)
            totals(r,t) = sum(solutions{r}.counts(t,:));
        end
    end
    handle = fopen(sprintf('%s\\Parameters.txt',experiment_label),'w+');
    fprintf(handle,paramString);
    outarray = zeros((length(solutions)+1)*(maxGens+1)+1,length(ts));
    outarray(1,:) = ts';
    for g = 1: maxGens
       means = zeros(length(ts),1);
       for r = 1: length(solutions)
            means=means+solutions{r}.counts(:,g);
       end
       outarray(g+1,:)=means(:)./length(solutions);
    end
    outarray(maxGens+2,:)=sum(outarray(2:g+1,:));
    for r = 1: length(solutions)
        for g = 1: maxGens
            outarray((r)*(maxGens+1)+g+1,:)=solutions{r}.counts(:,g);
        end
        outarray((r)*(maxGens+1)+maxGens+2,:)=totals(r,:);
    end
    csvwrite(sprintf('%s\\counts.csv',experiment_label),outarray');

    outarray = zeros(length(solutions)*maxGens*2,100);
    for r= 1: length(solutions)
        for g=1:maxGens
            outarray((r-1)*maxGens*2+g,:)=solutions{r}.qaverages(:,134,g);
        end
    end
    for r= 1: length(solutions)
        for g=1:maxGens
            outarray((r-1)*maxGens*2+g+maxGens,:)=solutions{r}.raverages(:,134,g);
        end
    end
    csvwrite(sprintf('%s\\Masses.csv',experiment_label),outarray');
    for r= 1: length(solutions)
        for g=1:maxGens
            outarray((r-1)*maxGens*2+g,:)=solutions{r}.qaverages(:,33,g);
        end
    end
    for r= 1: length(solutions)
        for g=1:maxGens
            outarray((r-1)*maxGens*2+g+maxGens,:)=solutions{r}.raverages(:,33,g);
        end
    end
    csvwrite(sprintf('%s\\RelA50n.csv',experiment_label),outarray');   
    for r= 1: length(solutions)
        for g=1:maxGens
            outarray((r-1)*maxGens*2+g,:)=solutions{r}.qaverages(:,53,g);
        end
    end
    for r= 1: length(solutions)
        for g=1:maxGens
            outarray((r-1)*maxGens*2+g+maxGens,:)=solutions{r}.raverages(:,53,g);
        end
    end
    csvwrite(sprintf('%s\\cRel50n.csv',experiment_label),outarray'); 
    for r= 1: length(solutions)
        for g=1:maxGens
            outarray((r-1)*maxGens*2+g,:)=solutions{r}.qaverages(:,94,g);
        end
    end
    for r= 1: length(solutions)
        for g=1:maxGens
            outarray((r-1)*maxGens*2+g+maxGens,:)=solutions{r}.raverages(:,94,g);
        end
    end
    csvwrite(sprintf('%s\\BclXL.csv',experiment_label),outarray'); 
    for r= 1: length(solutions)
        for g=1:maxGens
            outarray((r-1)*maxGens*2+g,:)=solutions{r}.qaverages(:,143,g);
        end
    end
    for r= 1: length(solutions)
        for g=1:maxGens
            outarray((r-1)*maxGens*2+g+maxGens,:)=solutions{r}.raverages(:,143,g);
        end
    end
    csvwrite(sprintf('%s\\Myc.csv',experiment_label),outarray'); 
    for r= 1: length(solutions)
        for g=1:maxGens
            outarray((r-1)*maxGens*2+g,:)=solutions{r}.qaverages(:,122,g);
        end
    end
    for r= 1: length(solutions)
        for g=1:maxGens
            outarray((r-1)*maxGens*2+g+maxGens,:)=solutions{r}.raverages(:,122,g);
        end
    end
    csvwrite(sprintf('%s\\CYCD.csv',experiment_label),outarray'); 
    outarray = zeros(maxGens,(length(solutions)+1)*9);
    for r = 1: length(solutions)
        for g = 1: maxGens
            outarray(g,(r)*9+1)=solutions{r}.fs(g);
            outarray(g,(r)*9+2)=solutions{r}.tp(g);
            outarray(g,(r)*9+3)=solutions{r}.fp(g);
            outarray(g,(r)*9+4)=solutions{r}.tn(g);
            outarray(g,(r)*9+5)=solutions{r}.fn(g);
            outarray(g,(r)*9+6)=solutions{r}.tp24(g);
            outarray(g,(r)*9+7)=solutions{r}.fp24(g);
            outarray(g,(r)*9+8)=solutions{r}.tn24(g);
            outarray(g,(r)*9+9)=solutions{r}.fn24(g);
            outarray(g, 1)=outarray(g, 1)+solutions{r}.fs(g)/length(solutions);
            outarray(g, 2)=outarray(g, 2)+solutions{r}.tp24(g)/length(solutions);
            outarray(g, 3)=outarray(g, 3)+solutions{r}.fp24(g)/length(solutions);
            outarray(g, 4)=outarray(g, 4)+solutions{r}.tn24(g)/length(solutions);
            outarray(g, 5)=outarray(g, 5)+solutions{r}.fn24(g)/length(solutions);
            outarray(g, 6)=outarray(g, 6)+(solutions{r}.tp24(g)/(solutions{r}.tp24(g)+solutions{r}.fp24(g)))/length(solutions);
            outarray(g, 7)=outarray(g, 7)+(solutions{r}.fp24(g)/(solutions{r}.tp24(g)+solutions{r}.fp24(g)))/length(solutions);
            outarray(g, 8)=outarray(g, 8)+(solutions{r}.tn24(g)/(solutions{r}.tn24(g)+solutions{r}.fn24(g)))/length(solutions);
            outarray(g, 9)=outarray(g, 9)+(solutions{r}.fn24(g)/(solutions{r}.tn24(g)+solutions{r}.fn24(g)))/length(solutions);

        end
    end
    csvwrite(sprintf('%s\\Fs.csv',experiment_label),outarray);    
    handle = fopen(sprintf('%s\\FateTimes.csv',experiment_label),'w+');
    for r = 1:length(solutions)
        for g = 1:maxGens
            fprintf(handle,',,%dTdec%d,,%dTdiv%d,,%dTdie%d',r,g,r,g,r,g);
        end
    end
    fprintf(handle,',AllTdec0,AllTdiv0,AllTdie0,AllTdec1+,AllTdiv1+,AllTdie1+\n');
    AllTdec0 = cell(1);
    AllTdiv0 = cell(1);
    AllTdie0 = cell(1);
    AllTdec1 = cell(1);
    AllTdiv1 = cell(1);
    AllTdie1 = cell(1);
    for r = 1: length(solutions)
        for t = 1:numel(solutions{r}.TdecideTimes{1})
            if numel(solutions{r}.TdecideTimes{1}{1}) > 0
                AllTdec0{end+1} = solutions{r}.TdecideTimes{1}{t};
            end
        end
        for t = 1:numel(solutions{r}.TdivTimes{1})
            if numel(solutions{r}.TdivTimes{1}{1}) > 0
                AllTdiv0{end+1} = solutions{r}.TdivTimes{1}{t};
            end
        end
        for t = 1:numel(solutions{r}.TdieTimes{1})
            if numel(solutions{r}.TdieTimes{1}{1}) > 0
                AllTdie0{end+1} = solutions{r}.TdieTimes{1}{t};
            end
        end

        for g = 2:maxGens
            for t = 1:numel(solutions{r}.TdecideTimes{g})
                if numel(solutions{r}.TdecideTimes{g}{t}) > 0
                    AllTdec1{end+1} = solutions{r}.TdecideTimes{g}{t};
                end
            end
            for t = 1:numel(solutions{r}.TdivTimes{g})
                if numel(solutions{r}.TdivTimes{g}{t}) > 0
                    AllTdiv1{end+1} = solutions{r}.TdivTimes{g}{t};
                end
            end
            for t = 1:numel(solutions{r}.TdieTimes{g})
                if numel(solutions{r}.TdieTimes{g}{t}) > 0
                    AllTdie1{end+1} = solutions{r}.TdieTimes{g}{t};
                end
            end
        end
    end
    for row = 1:100000
       rowcount=0;
       for r = 1: length(solutions)
            for i = 1:maxGens
                if row <= numel(solutions{r}.TdecideTimes{i}) 
                   fprintf(handle,',%d,%2.2f',solutions{r}.TdecideMothers{i}{row},solutions{r}.TdecideTimes{i}{row}/60.0);
                   rowcount=rowcount+1;
                else
                    fprintf(handle,',,');
                end
                if row <= numel(solutions{r}.TdivTimes{i})
                   fprintf(handle,',%d,%2.2f',solutions{r}.TdivMothers{i}{row},solutions{r}.TdivTimes{i}{row}/60.0);
                   rowcount=rowcount+1;
                else
                    fprintf(handle,',,');
                end
                if row <= numel(solutions{r}.TdieTimes{i})
                   fprintf(handle,',%d,%2.2f',solutions{r}.TdieMothers{i}{row},solutions{r}.TdieTimes{i}{row}/60.0);
                   rowcount=rowcount+1;
                else
                    fprintf(handle,',,');
                end
            end
       end
       if row <= numel(AllTdec0)
           fprintf(handle,',%2.2f',AllTdec0{row}/60.0);
           rowcount=rowcount+1;
       else
           fprintf(handle,',');
       end
       if row <= numel(AllTdiv0)
           fprintf(handle,',%2.2f',AllTdiv0{row}/60.0);
           rowcount=rowcount+1;
       else
           fprintf(handle,',');
       end
       if row <= numel(AllTdie0)
           fprintf(handle,',%2.2f',AllTdie0{row}/60.0);
           rowcount=rowcount+1;
       else
           fprintf(handle,',');
       end
       if row <= numel(AllTdec1)
           fprintf(handle,',%2.2f',AllTdec1{row}/60.0);
           rowcount=rowcount+1;
       else
           fprintf(handle,',');
       end
       if row <= numel(AllTdiv1)
           fprintf(handle,',%2.2f',AllTdiv1{row}/60.0);
           rowcount=rowcount+1;
       else
           fprintf(handle,',');
       end
       if row <= numel(AllTdie1)
           fprintf(handle,',%2.2f',AllTdie1{row}/60.0);
           rowcount=rowcount+1;
       else
           fprintf(handle,',');
       end

       fprintf(handle,'\n');
       if rowcount == 0
           break;
       end
    end
    fclose(handle);
    %csvwrite(sprintf('%s\\FateTimes.csv',experiment_label),[cell2mat(sol.TdecideTimes),cell2mat(sol.TdivTimes),cell2mat(sol.TdieTimes)]);
    %end
    %save Tdecides, Tdivs, Tides 
    %fprintf(1,'Saved all masses and counts to %s\n',experiment_label);
end

% Make a copy of a handle object.
function new = copy(this)
    % Instantiate new object of the same class.
    new = feval(class(this));
    % Copy all non-hidden properties.
    p = fieldnames(struct(this));
    for i = 1:length(p)
        new.(p{i}) = this.(p{i});
    end
end

function c2 = duplicate(sol)
    c2 = struct;   
    c2.NFKB_CYCD_T=sol.NFKB_CYCD_T;
    c2.NFKB_BCL2_T=sol.NFKB_BCL2_T;
    c2.NFKB_MYC_T=sol.NFKB_MYC_T;
    c2.GROWTH_T=sol.GROWTH_T;
    c2.RATE_CV=sol.RATE_CV;
    c2.CONC_CV=sol.CONC_CV;
    c2.PARTITION_CV=sol.PARTITION_CV;
    c2.VOL_CV=sol.VOL_CV;
    c2.IKK_CV=sol.IKK_CV;
    c2.GMSyn=sol.GMSyn;
    c2.GMDeg=sol.GMDeg;
    c2.MassGrowth=sol.MassGrowth;
    c2.MassDecay=sol.MassDecay;
    c2.CYCDtSyn=    sol.CYCDtSyn;
    c2.CYCDtDeg=    sol.CYCDtDeg;
    c2.BCL2tSyn=    sol.BCL2tSyn;
    c2.BCL2tDeg=	sol.BCL2tDeg;
    c2.MYCSyn=      sol.MYCSyn;
    c2.MYCDeg=      sol.MYCDeg;
    c2.MYCtSyn=     sol.MYCtSyn;
    c2.MYCtDeg=     sol.MYCtDeg;
    c2.L=           sol.L;
    c2.STIMULUS=    sol.STIMULUS;
    c2.CREL=        sol.CREL;
    c2.cRelFeedback=sol.cRelFeedback;
    c2.EPS=         sol.EPS;
    c2.TOTAL_IKK=   sol.TOTAL_IKK;
    c2.maxTime=sol.maxTime;  
end



function v = getRandomCell(v)
        %--- Randomization of NFkB syn/deg params
        for i = [1,2,3,4,5,11,12,13,14]          
            for j = 1:size(v.pi(i,:),2)
                v.pi(i,j)=max(0,normrnd(v.pi(i,j),v.pi(i,j)*v.RATE_CV));
            end
        end
        for i = [1,2,3,4,5]         
            for j = 1:size(v.pm(i,:),2)
                v.pm(i,j)=max(0,normrnd(v.pm(i,j),v.pm(i,j)*v.RATE_CV));
            end
        end
        for i = 9:12          
            for j = 1:size(v.pd(i,:),2)
                v.pd(i,j)=max(0,normrnd(v.pd(i,j),v.pd(i,j)*v.RATE_CV));
            end
        end
        %--- apoptosis syn/deg
        for i = 71:115
            v.k(i)= max(0,normrnd(v.k(i),v.k(i)*v.RATE_CV));
        end
        %--- cell cycle syn/deg
        for i = [8,9,12:15,18:34,47:67,74:76,78:86]
            v.k2(i)= max(0,normrnd(v.k2(i),v.k2(i)*v.RATE_CV));
        end
        %--- cell cycle species randomization
        theMask = zeros(size(v.SPECIES{1},1),1);
        theMask(137:142) = 1;
        v.SPECIES{1} = getRandomSpeciesMaskLognormal(v.SPECIES{1},theMask,v.CONC_CV,0);
end


function v2 = getDaughterCell(v,vo,r)
        v2 = copy(v);
        for i = [1,2,3,4,5,11,12,13,14]          
            for j = 1:size(v.pi(i,:),2)
                v2.pi(i,j)=max(0,(normrnd(v.pi(i,j),v.pi(i,j)*v.RATE_CV)+normrnd(vo.pi(i,j),vo.pi(i,j)*vo.RATE_CV))/2);
            end
        end
        for i = [1,2,3,4,5]         
            for j = 1:size(v.pm(i,:),2)
                v2.pm(i,j)=max(0,(normrnd(v.pm(i,j),v.pm(i,j)*v.RATE_CV)+normrnd(vo.pm(i,j),vo.pm(i,j)*vo.RATE_CV))/2);
            end
        end
        for i = 9:12          
            for j = 1:size(v.pd(i,:),2)
                v2.pd(i,j)=max(0,(normrnd(v.pd(i,j),v.pd(i,j)*v.RATE_CV)+normrnd(vo.pd(i,j),vo.pd(i,j)*vo.RATE_CV))/2);
            end
        end
        %--- apoptosis syn/deg
        for i = 71:115
            v2.k(i,j)=max(0,(normrnd(v.k(i),v.k(i)*v.RATE_CV)+normrnd(vo.k(i),vo.k(i)*vo.RATE_CV))/2);
        end
        %--- cell cycle syn/deg
        for i = [8,9,12:15,18:34,47:67,74:76,78:86]
            v2.k2(i,j)=max(0,(normrnd(v.k2(i),v.k2(i)*v.RATE_CV)+normrnd(vo.k2(i),vo.k2(i)*vo.RATE_CV))/2);
        end
        v2.SPECIES{1}(132)=v.SPECIES{1}(132)*r;
        v2.SPECIES{1}(134)=v.SPECIES{1}(134)*r;
        %==== IKK Input Curve
        if vo.IKK_CV > 0
            sigma= sqrt(log(1+(vo.IKK_CV^2)));
            mu= log(140)-0.5*sigma^2; 
            rnd = lognrnd(mu,sigma);
            v2.TOTAL_IKK=(v.TOTAL_IKK+rnd)/2;
        end
end

     % NFKB_CYCD_T NFKB_BCL2_T  NFKB_MYC_T    GROWTH_T  
     % RATE_CV CONC_CV PARTITION_CV VOL_CV IKK_CV  
     % GMSyn  GMDeg  MassGrowth MassDecay 
     % CYCDtSyn CYCDtDeg BCL2Syn BCL2tDeg 
     % MYCSyn MYCDeg mMycSyn mMycDeg L
     % IKK STIMULUS, Crel monomer,cRelFeedback,eps (Ribosome modifier);IKKt
function c2= randomCondition(maxTime,parameters)
    c2 = struct;
    c2.NFKB_CYCD_T= parameters(1);
    c2.NFKB_BCL2_T= parameters(2);
    c2.NFKB_MYC_T=  parameters(3);
    c2.GROWTH_T=    parameters(4);
    c2.RATE_CV=     parameters(5);
    c2.CONC_CV=     parameters(6);
    c2.PARTITION_CV=parameters(7);
    c2.VOL_CV=      parameters(8);
    c2.IKK_CV=      parameters(9);
    c2.GMSyn=       parameters(10);
    c2.GMDeg=       parameters(11);
    c2.MassGrowth=  parameters(12);
    c2.MassDecay=   parameters(13);
    c2.CYCDtSyn=    parameters(14);
    c2.CYCDtDeg=    parameters(15);
    c2.BCL2tSyn=    parameters(16);
    c2.BCL2tDeg=	parameters(17);
    c2.MYCSyn=      parameters(18);
    c2.MYCDeg=      parameters(19);
    c2.MYCtSyn=     parameters(20);
    c2.MYCtDeg=     parameters(21);
    c2.L=           parameters(22);
    c2.STIMULUS=    parameters(23);
    c2.CREL=        parameters(24);
    c2.cRelFeedback=parameters(25);
    c2.EPS=         parameters(26);
    c2.TOTAL_IKK   =parameters(27);
    c2.maxTime=maxTime;
end


function [kinit]=getRandomRatesMaskNormal(k0,mask,CV)
    kinit = zeros(length(k0),1);
    for cc=1:length(k0)
        factor = 1;
        if mask(cc) == 1
            factor = normrnd(1,double(sqrt(CV)));
        end
        kinit(cc)=factor*k0(cc);
    end
end

function [x0aa]= getRandomSpeciesMaskLognormal(x0,mask,CV,GaussianNoise)
    x0aa = zeros(length(x0),1);
    for cc=1:length(x0)
        factor = 1;
        if GaussianNoise > 0
            factor = normrnd(1,double(GaussianNoise));
        end
        if CV >0
            sigma = sqrt(log(CV*CV+1));
        else
            sigma = 0;
        end
        mu= log(max(0,x0(cc))); 
        if mask(cc) == 1
            rnd = lognrnd(mu,sigma);
            if mu==0 
                rnd=0;
            end
            x0aa(cc)=rnd*factor;
         
        else
            x0aa(cc)=x0(cc)*factor;
        end
    end
end


